//
//  MasterViewController.swift
//  BWSwipeTableCellExample
//
//  Created by Kyle Newsome on 2015-11-12.
//  Copyright © 2015 Kyle Newsome. All rights reserved.
//

import UIKit
import CoreData
import BWSwipeRevealCell

class MasterViewController: UITableViewController, NSFetchedResultsControllerDelegate, BWSwipeRevealCellDelegate {
    
    var managedObjectContext: NSManagedObjectContext? = nil
    
    override func viewDidLoad() {
        super.viewDidLoad()
        let addButton = UIBarButtonItem(barButtonSystemItem: .Add, target: self, action: "insertNewObject:")
        self.navigationItem.rightBarButtonItem = addButton
    }
    
    func insertNewObject(sender: AnyObject) {
        let context = self.fetchedResultsController.managedObjectContext
        let entity = self.fetchedResultsController.fetchRequest.entity!
        
        //Create one of each type
        for i in 0..<3 {
            let newManagedObject = NSEntityDescription.insertNewObjectForEntityForName(entity.name!, inManagedObjectContext: context)
            newManagedObject.setValue(i, forKey: "type")
        }
        
        do {
            try context.save()
        } catch {
            abort()
        }
    }
    
    func removeObjectAtIndexPath(indexPath:NSIndexPath) {
        let context = self.fetchedResultsController.managedObjectContext
        //Deleting objects regardless of done/delete for the purpose of this example
        context.deleteObject(self.fetchedResultsController.objectAtIndexPath(indexPath) as! NSManagedObject)
        do {
            try context.save()
        } catch {
            abort()
        }
    }
    
    // MARK: - Table View
    
    override func tableView(tableView: UITableView, heightForRowAtIndexPath indexPath: NSIndexPath) -> CGFloat {
        return 72
    }
    
    override func numberOfSectionsInTableView(tableView: UITableView) -> Int {
        return self.fetchedResultsController.sections?.count ?? 0
    }
    
    override func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        let sectionInfo = self.fetchedResultsController.sections![section]
        return sectionInfo.numberOfObjects
    }
    
    override func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCellWithIdentifier("Cell", forIndexPath: indexPath)
        self.configureCell(cell, atIndexPath: indexPath)
        return cell
    }
    
    func configureCell(cell: UITableViewCell, atIndexPath indexPath: NSIndexPath) {
        let object = self.fetchedResultsController.objectAtIndexPath(indexPath)
        let swipeCell:BWSwipeRevealCell = cell as! BWSwipeRevealCell
        
        swipeCell.bgViewLeftImage = UIImage(named:"Done")!.imageWithRenderingMode(.AlwaysTemplate)
        swipeCell.bgViewLeftColor = UIColor.greenColor()
        
        swipeCell.bgViewRightImage = UIImage(named:"Delete")!.imageWithRenderingMode(.AlwaysTemplate)
        swipeCell.bgViewRightColor = UIColor.redColor()
        
        let type = BWSwipeCellType(rawValue: object.valueForKey("type") as! Int)!
        swipeCell.type = type
        
        switch type {
        case .SwipeThrough:
            swipeCell.textLabel!.text = "Swipe Through"
            break
        case .SpringRelease:
            swipeCell.textLabel!.text = "Spring Release"
            break
        case .SlidingDoor:
            swipeCell.textLabel!.text = "Sliding Door"
            break
        }
        
        swipeCell.delegate = self
    }
    
    // MARK: - Fetched results controller
    
    var fetchedResultsController: NSFetchedResultsController {
        if _fetchedResultsController != nil {
            return _fetchedResultsController!
        }
        
        let fetchRequest = NSFetchRequest()
        let entity = NSEntityDescription.entityForName("Event", inManagedObjectContext: self.managedObjectContext!)
        fetchRequest.entity = entity
        fetchRequest.fetchBatchSize = 20
        let sortDescriptor = NSSortDescriptor(key: "type", ascending: false)
        fetchRequest.sortDescriptors = [sortDescriptor]
        let aFetchedResultsController = NSFetchedResultsController(fetchRequest: fetchRequest, managedObjectContext: self.managedObjectContext!, sectionNameKeyPath: nil, cacheName: "Master")
        aFetchedResultsController.delegate = self
        _fetchedResultsController = aFetchedResultsController
        
        do {
            try _fetchedResultsController!.performFetch()
        } catch {
            abort()
        }
        
        return _fetchedResultsController!
    }
    var _fetchedResultsController: NSFetchedResultsController? = nil
    
    func controllerWillChangeContent(controller: NSFetchedResultsController) {
        self.tableView.beginUpdates()
    }
    
    func controller(controller: NSFetchedResultsController, didChangeSection sectionInfo: NSFetchedResultsSectionInfo, atIndex sectionIndex: Int, forChangeType type: NSFetchedResultsChangeType) {
        switch type {
        case .Insert:
            self.tableView.insertSections(NSIndexSet(index: sectionIndex), withRowAnimation: .Fade)
        case .Delete:
            self.tableView.deleteSections(NSIndexSet(index: sectionIndex), withRowAnimation: .Fade)
        default:
            return
        }
    }
    
    func controller(controller: NSFetchedResultsController, didChangeObject anObject: AnyObject, atIndexPath indexPath: NSIndexPath?, forChangeType type: NSFetchedResultsChangeType, newIndexPath: NSIndexPath?) {
        switch type {
        case .Insert:
            tableView.insertRowsAtIndexPaths([newIndexPath!], withRowAnimation: .Fade)
        case .Delete:
            tableView.deleteRowsAtIndexPaths([indexPath!], withRowAnimation: .Fade)
        case .Update:
            self.configureCell(tableView.cellForRowAtIndexPath(indexPath!)!, atIndexPath: indexPath!)
        case .Move:
            tableView.deleteRowsAtIndexPaths([indexPath!], withRowAnimation: .Fade)
            tableView.insertRowsAtIndexPaths([newIndexPath!], withRowAnimation: .Fade)
        }
    }
    
    func controllerDidChangeContent(controller: NSFetchedResultsController) {
        self.tableView.endUpdates()
    }
    
    // MARK: - Reveal Cell Delegate
    
    func swipeCellWillRelease(cell: BWSwipeCell) {
        print("Swipe Cell Will Release")
        if cell.state != .Normal && cell.type != .SlidingDoor {
            let indexPath: NSIndexPath = tableView.indexPathForCell(cell)!
            self.removeObjectAtIndexPath(indexPath)
        }
    }
    
    func swipeCellActivatedAction(cell: BWSwipeCell, isActionLeft: Bool) {
        print("Swipe Cell Activated Action")
        let indexPath: NSIndexPath = tableView.indexPathForCell(cell)!
        self.removeObjectAtIndexPath(indexPath)
    }
    
    func swipeCellDidChangeState(cell: BWSwipeCell) {
        print("Swipe Cell Did Change State")
        if cell.state != .Normal {
            print("-> Cell Passed Threshold")
        } else {
            print("-> Cell Returned to Normal")
        }
    }
    
    func swipeCellDidCompleteRelease(cell: BWSwipeCell) {
        print("Swipe Cell Did Complete Release")
    }
    
    func swipeCellDidSwipe(cell: BWSwipeCell) {
        print("Swipe Cell Did Swipe")
    }
    
    func swipeCellDidStartSwiping(cell: BWSwipeCell) {
        print("Swipe Cell Did Start Swiping")
    }
    
}

